package com.mbeers;

import java.util.List;

import android.content.Context;
import android.os.Build.VERSION;
import android.view.View;

import com.mbeers.events.KeyEvent;
import com.mbeers.events.TouchEvent;
import com.mbeers.handlers.AccelerometerHandler;
import com.mbeers.handlers.KeyboardHandler;
import com.mbeers.handlers.MultiTouchHandler;
import com.mbeers.handlers.SingleTouchHandler;
import com.mbeers.interfaces.Input;
import com.mbeers.interfaces.Touch;

/**
 * Android user input.
 * 
 * @author Michael Beers
 * @todo Docblocks
 */
public class AndroidInput implements Input {

	AccelerometerHandler accelHandler;
	KeyboardHandler keyHandler;
	Touch touchHandler;

	// ----------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param context
	 * @param view
	 * @param scaleX
	 * @param scaleY
	 */
	public AndroidInput(Context context, View view, float scaleX, float scaleY) {
		accelHandler = new AccelerometerHandler(context);
		keyHandler = new KeyboardHandler(view);

		if (VERSION.SDK_INT < 5) {
			touchHandler = new SingleTouchHandler(view, scaleX, scaleY);
		} else {
			touchHandler = new MultiTouchHandler(view, scaleX, scaleY);
		}
	}

	// ----------------------------------------

	@Override
	public boolean isKeyPressed(int keyCode) {
		return keyHandler.isKeyPressed(keyCode);
	}

	@Override
	public boolean isTouchDown(int pointer) {
		return touchHandler.isTouchDown(pointer);
	}

	@Override
	public int getTouchX(int pointer) {
		return touchHandler.getTouchX(pointer);
	}

	@Override
	public int getTouchY(int pointer) {
		return touchHandler.getTouchY(pointer);
	}

	@Override
	public float getAccelX() {
		return accelHandler.getAccelX();
	}

	@Override
	public float getAccelY() {
		return accelHandler.getAccelY();
	}

	@Override
	public float getAccelZ() {
		return accelHandler.getAccelZ();
	}

	@Override
	public List<KeyEvent> getKeyEvents() {
		return keyHandler.getKeyEvents();
	}

	@Override
	public List<TouchEvent> getTouchEvents() {
		return touchHandler.getTouchEvents();
	}

}
