package com.mbeers;

import java.util.ArrayList;
import java.util.List;

import com.mbeers.interfaces.PoolObjectFactory;

/**
 * Pool.
 * 
 * This is the garbage collector.
 * 
 * @author Michael Beers
 * @todo Docblocks
 */
public class Pool<T> {

	private final List<T> freeObjects;
	private final PoolObjectFactory<T> factory;
	private final int maxSize;

	// ----------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param factory
	 * @param maxSize
	 */
	public Pool(PoolObjectFactory<T> factory, int maxSize) {
		this.factory = factory;
		this.maxSize = maxSize;
		this.freeObjects = new ArrayList<T>(maxSize);
	}

	/**
	 * Create a new instance of a pool object.
	 * 
	 * @return
	 */
	public T newObject() {
		T object = null;

		if (freeObjects.isEmpty()) {
			object = factory.createObject();
		} else {
			object = freeObjects.remove(freeObjects.size() - 1);
		}

		return object;
	}

	/**
	 * Reinsert a pool object that is no longer in use.
	 * 
	 * @param object
	 */
	public void free(T object) {
		if (freeObjects.size() < maxSize) {
			freeObjects.add(object);
		}
	}
	
}
