package com.mbeers;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

/**
 * Android Fast render view.
 * 
 * No description yet. TODO Docblocks
 * 
 * @author Michael Beers
 */
@SuppressLint("ViewConstructor")
public class AndroidFastRenderView extends SurfaceView implements Runnable {

	AndroidGame game;
	Bitmap framebuffer;
	Thread renderThread = null;
	SurfaceHolder holder;
	volatile boolean running = false;

	// ----------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param game
	 * @param framebuffer
	 */
	public AndroidFastRenderView(AndroidGame game, Bitmap framebuffer) {
		super(game);
		this.game = game;
		this.framebuffer = framebuffer;
		this.holder = getHolder();
	}

	// ----------------------------------------

	@Override
	public void run() {
		Rect dstRect = new Rect();
		long startTime = System.nanoTime();
		while (running) {
			if (!holder.getSurface().isValid()) {
				continue;
			}

			float deltaTime = (System.nanoTime() - startTime) / 10000000.000f;
			startTime = System.nanoTime();

			if (deltaTime > 3.15) {
				deltaTime = (float) 3.15;
			}

			game.getCurrentScreen().update(deltaTime);
			game.getCurrentScreen().render(deltaTime);

			Canvas canvas = holder.lockCanvas();
			canvas.getClipBounds(dstRect);
			canvas.drawBitmap(framebuffer, null, dstRect, null);
			holder.unlockCanvasAndPost(canvas);
		}
	}

	/**
	 * Pauses render thread.
	 */
	public void pause() {
		running = false;
		while (true) {
			try {
				renderThread.join();
				break;
			} catch (InterruptedException e) {
				// retry
			}
		}
	}

	/**
	 * Resumes render thread.
	 */
	public void resume() {
		running = true;
		renderThread = new Thread(this);
		renderThread.start();
	}

}
