package com.mbeers;

import java.util.ArrayList;
import java.util.List;

/**
 * Pool.
 * 
 * Factory to collect all objects.
 * 
 * @author Michael Beers
 */
public class Pool<T> {

	/**
	 * A single pool object.
	 * 
	 * @param <T>
	 */
	public interface PoolObjectFactory<T> {
		public T createObject();
	}

	private final List<T> freeObjects;
	private final PoolObjectFactory<T> factory;
	private final int maxSize;

	// ----------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param factory
	 * @param maxSize
	 */
	public Pool(PoolObjectFactory<T> factory, int maxSize) {
		this.factory = factory;
		this.maxSize = maxSize;
		this.freeObjects = new ArrayList<T>(maxSize);
	}

	// ----------------------------------------

	/**
	 * Create a new pool object.
	 * 
	 * @return
	 */
	public T newObject() {
		T object = null;

		if (freeObjects.size() == 0) {
			object = factory.createObject();
		} else {
			object = freeObjects.remove(freeObjects.size() - 1);
		}

		return object;
	}

	/**
	 * Add a new pool object.
	 * 
	 * @param object
	 */
	public void free(T object) {
		if (freeObjects.size() < maxSize)
			freeObjects.add(object);
	}

}
