package com.mbeers;

import com.mbeers.interfaces.Game;

/**
 * Screen.
 * 
 * Factory for displaying or changing game screens.
 * 
 * Note: This class acts like an singleton.
 * 
 * @author Michael Beers
 */
public abstract class Screen {

	/**
	 * Instance of the game, so this class acts like an singleton.
	 */
	protected final Game game;

	// ----------------------------------------

	/**
	 * Constructor.
	 * 
	 * @param game
	 */
	public Screen(Game game) {
		this.game = game;
	}

	// ----------------------------------------
	
	/**
	 * Updates the current screen.
	 * 
	 * @param deltaTime
	 */
	public abstract void update(float deltaTime);
	
	/**
	 * Renders the current screen.
	 * 
	 * @param deltaTime
	 */
	public abstract void render(float deltaTime);
	
	/**
	 * Pauses the current game screen.
	 */
	public abstract void pause();

	/**
	 * Resumes the current game screen.
	 */
	public abstract void resume();
	
	/**
	 * Disposes/destroy the current game screen.
	 */
	public abstract void dispose();
	
}
