package com.mbeers.interfaces;

import com.mbeers.Screen;

/**
 * Game interface.
 * 
 * Ties every other interfaces together.
 * 
 * @author Michael Beers
 */
public interface Game {

	/**
	 * Get the audio interface.
	 * 
	 * @return
	 */
	public Audio getAudio();

	/**
	 * Get the user input interface.
	 * 
	 * @return
	 */
	public Input getInput();

	/**
	 * Get the file input/output interface.
	 * 
	 * @return
	 */
	public FileIO getFileIO();

	/**
	 * Get the graphic interface.
	 * 
	 * @return
	 */
	public Graphics getGraphics();

	/**
	 * Set the game screen.
	 * 
	 * @param screen
	 */
	public void setScreen(Screen screen);

	/**
	 * Get the current game screen.
	 * 
	 * @return
	 */
	public Screen getCurrentScreen();

	/**
	 * Get the start screen of the game.
	 * 
	 * @return
	 */
	public Screen getStartScreen();

}
