package com.mbeers.interfaces;

import android.graphics.Paint;

/**
 * Graphics interface.
 * 
 * Ties every graphic based interface together.
 * 
 * @author Michael Beers
 */
public interface Graphics {

	/**
	 * List with image formats.
	 */
	public static enum ImageFormat {
		ARGB8888, ARGB4444, RGB565
	}

	/**
	 * Create a new image.
	 * 
	 * @param fileName
	 * @param format
	 * @return
	 */
	public Image createImage(String fileName, ImageFormat format);

	/**
	 * Fill the screen with an solid color.
	 * 
	 * @param color
	 */
	public void clearScreen(int color);

	/**
	 * Draw a line on the canvas.
	 * 
	 * @param x
	 * @param y
	 * @param x2
	 * @param y2
	 * @param color
	 */
	public void drawLine(int x, int y, int x2, int y2, int color);

	/**
	 * Draw a rectangle on the canvas.
	 * 
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param color
	 */
	public void drawRect(int x, int y, int width, int height, int color);

	/**
	 * Draw a image on the canvas.
	 * 
	 * Note: Use the srcX, srcY, srcWidth, srcHeight to render just an part of
	 * the image, easy for animations or UI's.
	 * 
	 * @param image
	 * @param x
	 * @param y
	 * @param srcX
	 * @param srcY
	 * @param srcWidth
	 * @param srcHeight
	 */
	public void drawImage(Image image, int x, int y, int srcX, int srcY,
			int srcWidth, int srcHeight);

	/**
	 * Draw a image on the canvas.
	 * 
	 * Note: Use the srcX, srcY, srcWidth, srcHeight to render just an part of
	 * the image, easy for animations or UI's.
	 * 
	 * @param image
	 * @param x
	 * @param y
	 */
	public void drawImage(Image image, int x, int y);

	/**
	 * Draw a string on the canvas.
	 * 
	 * @param text
	 * @param x
	 * @param y
	 * @param paint
	 */
	public void drawString(String text, int x, int y, Paint paint);

	/**
	 * Get width.
	 * 
	 * @return
	 */
	public int getWidth();

	/**
	 * Get height.
	 * 
	 * @return
	 */
	public int getHeight();

	/**
	 * Draw ARGB.
	 * 
	 * TODO Better documentation.
	 * 
	 * @param a
	 * @param r
	 * @param g
	 * @param b
	 */
	public void drawARGB(int a, int r, int g, int b);

}
