package com.mbeers.interfaces;

import java.util.List;

/**
 * User input interface.
 * 
 * Interface that handles all user input.
 * 
 * @author Michael Beers
 */
public interface Input {

	/**
	 * A single touch event.
	 */
	public static class TouchEvent {
		public static final int TOUCH_DOWN = 0;
		public static final int TOUCH_UP = 1;
		public static final int TOUCH_DRAGGED = 2;
		public static final int TOUCH_HOLD = 3;

		public int type;
		public int x, y;
		public int pointer;
	}

	/**
	 * Get whether the users finger is touched on the screen.
	 * 
	 * @param pointer
	 * @return
	 */
	public boolean isTouchDown(int pointer);

	/**
	 * Get X-coordinate of the touch event.
	 * 
	 * @param pointer
	 * @return
	 */
	public int getTouchX(int pointer);

	/**
	 * Get Y-coordinate of the touch event.
	 * 
	 * @param pointer
	 * @return
	 */
	public int getTouchY(int pointer);

	/**
	 * Get all touch events.
	 * 
	 * @return
	 */
	public List<TouchEvent> getTouchEvents();

}
