package com.mbeers.interfaces;

/**
 * Music interface.
 * 
 * The interface for large audio files e.q. background music.
 * 
 * @author Michael Beers
 */
public interface Music {

	/**
	 * Play the sound.
	 */
	public void play();

	/**
	 * Stop the sound.
	 */
	public void stop();

	/**
	 * Pauses the sound.
	 */
	public void pause();

	/**
	 * Set the repeat-mode on/off of the sound.
	 * 
	 * @param looping
	 */
	public void setLooping(boolean looping);

	/**
	 * Set the volume of the sound.
	 * 
	 * @param volume
	 */
	public void setVolume(float volume);

	/**
	 * Get whether the sound is playing.
	 * 
	 * @return
	 */
	public boolean isPlaying();

	/**
	 * Get whether the sound is stopped.
	 * 
	 * @return
	 */
	public boolean isStopped();

	/**
	 * Gets whether the sound is repeating.
	 * 
	 * @return
	 */
	public boolean isLooping();

	/**
	 * Disposes/destroy sound.
	 */
	public void dispose();

	/**
	 * Search for the begin of the sound.
	 */
	void seekBegin();

}
